//
//  QHVCNet.h
//  QHVCNet
//
//  Created by yangkui on 2017/8/29.
//  Copyright © 2017年 yangkui. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "QHVCNetEnumerates.h"
#import <QHVCCommonKit/QHVCCommonKit.h>

NS_ASSUME_NONNULL_BEGIN

/**
 * 需要开启服务的KEY，值见：QHVCNetServiceType。默认值：QHVCNetServicePrecache
 * 如果需要同时开启多个服务，多个服务的值相加即可，例如：QHVCNetServicePrecache|QHVCNetServiceGodSees
 */
extern NSString* const kQHVCNetOptionsServicesKey;//开启服务类型

#pragma mark - SDK具体实现 -

@interface QHVCNet: NSObject

#pragma mark - 基础功能 -

/**
 该方法生成一个单例QHVCNet
 
 @return QHVCNet对象
 */
+ (instancetype)sharedInstance;

/**
 获取QHVCNetKit的版本信息

 @return 字符串版本号
 */
- (NSString *)getVersion;

/// 设置日志等级（控制台输出）
/// @param logLevel 控制台输出日志等级
- (void)setLogLevel:(QHVCLogLevel)logLevel;

/// 设置日志输出回调，供业务调试使用（仅供开发阶段）
/// @param callback 回调block
- (void)setLogOutputCallBack:(void(^)(QHVCLogLevel level, NSString *data))callback;

/**
 开启LocalServer， 全局只用调用一次， 需要和stopLocalServer配对使用
 
 @param cacheDir 缓存目录，要保证是一个当前存在且有读写权限的文件夹
 @param cacheSizeInMB 初始缓存大小 单位：MB
 @param options @{kQHVCNetOptionsServicesKey:@(QHVCNetServicePrecache | QHVCNetServiceGodsees),
                  kQHVCNetGodSeesOptionsUseLongConnectKey:@(QHVCNetLongConnectService)
                 }
 @param completion 初始化完成回调
 */
- (void)startLocalServer:(NSString *)cacheDir
               cacheSize:(int)cacheSizeInMB
                 options:(NSDictionary *)options
              completion:(void(^)(BOOL result, NSError* error))completion;

/**
 停止本地服务， 需要和startLocalServer配对使用
 */
- (void)stopLocalServer;

/**
 获取localserver播放的链接
 
 @param rid 资源的唯一标识，由于url在加防盗链时是可变的，所以需要一个唯一标识来匹配缓存文件
 @param url 资源的链接
 @return 走预缓存播放的链接，不成功将会返回空
 */
- (NSString *)getLocalServerPlayUrl:(NSString *)rid url:(NSString *)url;

/**
 获取localserver播放的链接
 
 @param rid 资源的唯一标识，由于url在加防盗链时是可变的，所以需要一个唯一标识来匹配缓存文件
 @param url 资源的链接
 @param options @{@"open_p2p":@"boolValue",@"open_localserver":@"boolValue"}
 @return 走预缓存播放的链接，不成功将会返回空
 */
- (NSString *)getLocalServerPlayUrl:(NSString *)rid url:(NSString *)url options:(NSDictionary * _Nullable)options;

/// 获取localserver播放的链接
/// 启用socks5代理，就不再考虑p2p地址和localserver地址组合使用
/// @param rid 资源的唯一标识，由于url在加防盗链时是可变的，所以需要一个唯一标识来匹配缓存文件
/// @param url 资源的链接
/// @param socks5ProxyIp socks5代理服务器ip地址
/// @param socks5ProxyPort socks5代理服务器端口号
/// @param socks5ProxyHeader socks5代理服务器自定义header，用于代理服务器鉴权,格式样例 key1: valule1 \r\nkey2: valule2 \r\n
- (NSString *)getLocalServerPlayUrl:(NSString *)rid
                                url:(NSString *)url
                      socks5ProxyIp:(NSString *)socks5ProxyIp
                    socks5ProxyPort:(int)socks5ProxyPort
                  socks5ProxyHeader:(NSString *)socks5ProxyHeader;

/**
 是否开启LocalServer
 
 @return YES 开启 NO 未开启
 */
- (BOOL)isStartLocalServer;

/**
 设置和调整缓存占用空间大小。 这个接口可以中途调用，可以调用多次
 
 @param cacheSizeInMB 缓存空间的大小， 单位:MB
 @return yes:成功 no:失败
 */
- (BOOL)setCacheSize:(int)cacheSizeInMB;

/**
 * 获取网络状态
 */
- (QHVCNetNetworkStatus)networkStatus;

/**
 恢复/禁止QHVCNetKit的所有访问网络主动拉取数据的行为
 
 @param enable YES:允许，NO：禁止，默认：允许
 */
- (void)enableNetwork:(BOOL)enable;

/**
 是否允许QHVCNetKit所有访问网络主动拉取数据的行为
 
 @return YES：允许，NO：禁止
 */
- (BOOL)isEnableNetwork;

/**
 获取p2p信息
 
 @return 详情
 */
- (nullable NSDictionary *)getP2pInfo;

@end

NS_ASSUME_NONNULL_END
